"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.filterSyncedFields = exports.FilterSyncedFieldsError = void 0;
const trimop_1 = require("trimop");
const data_1 = require("./data");
function FilterSyncedFieldsError(field) {
    return {
        field,
    };
}
exports.FilterSyncedFieldsError = FilterSyncedFieldsError;
/**
 * Filter synced fields.
 * @param doc to be filtered
 * @param syncedFields
 * @returns synced fields if exists, undefined otherwise.
 */
function filterSyncedFields({ doc, syncedFields, }) {
    return trimop_1.eitherArrayReduce(Object.entries(syncedFields), trimop_1.Right(trimop_1.None()), (acc, [fieldName, syncFieldSpec]) => {
        return trimop_1.optionFold(trimop_1.optionFromNullable(doc[fieldName]), () => trimop_1.Right(acc), (field) => {
            // Copy the field if defined in the spec
            if (syncFieldSpec === true) {
                const fieldEntry = { [fieldName]: field };
                return trimop_1.Right(trimop_1.Some(trimop_1.optionFold(acc, () => fieldEntry, (acc) => (Object.assign(Object.assign({}, acc), fieldEntry)))));
            }
            if (field._type !== 'Ref') {
                return trimop_1.Left(FilterSyncedFieldsError(field));
            }
            // Copy nested synced fields
            return trimop_1.eitherMapRight(filterSyncedFields({
                doc: field.snapshot.doc,
                syncedFields: syncFieldSpec,
            }), (syncedDoc) => trimop_1.Right(trimop_1.optionFold(syncedDoc, 
            // If there was no copied field
            () => acc, (syncedDoc) => trimop_1.optionFold(acc, () => trimop_1.Some({
                [fieldName]: data_1.RefField({
                    doc: syncedDoc,
                    id: field.snapshot.id,
                }),
            }), (acc) => trimop_1.Some(Object.assign(Object.assign({}, acc), { [fieldName]: data_1.RefField({
                    doc: syncedDoc,
                    id: field.snapshot.id,
                }) }))))));
        });
    });
}
exports.filterSyncedFields = filterSyncedFields;
//# sourceMappingURL=filter-synced-fields.js.map