var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { eitherArrayReduce, eitherMapRight, Left, None, optionFold, optionFromNullable, Right, Some, } from 'trimop';
import { RefField } from './data';
export function FilterSyncedFieldsError(field) {
    return {
        field: field,
    };
}
/**
 * Filter synced fields.
 * @param doc to be filtered
 * @param syncedFields
 * @returns synced fields if exists, undefined otherwise.
 */
export function filterSyncedFields(_a) {
    var doc = _a.doc, syncedFields = _a.syncedFields;
    return eitherArrayReduce(Object.entries(syncedFields), Right(None()), function (acc, _a) {
        var fieldName = _a[0], syncFieldSpec = _a[1];
        return optionFold(optionFromNullable(doc[fieldName]), function () { return Right(acc); }, function (field) {
            var _a;
            // Copy the field if defined in the spec
            if (syncFieldSpec === true) {
                var fieldEntry_1 = (_a = {}, _a[fieldName] = field, _a);
                return Right(Some(optionFold(acc, function () { return fieldEntry_1; }, function (acc) { return (__assign(__assign({}, acc), fieldEntry_1)); })));
            }
            if (field._type !== 'Ref') {
                return Left(FilterSyncedFieldsError(field));
            }
            // Copy nested synced fields
            return eitherMapRight(filterSyncedFields({
                doc: field.snapshot.doc,
                syncedFields: syncFieldSpec,
            }), function (syncedDoc) {
                return Right(optionFold(syncedDoc, 
                // If there was no copied field
                function () { return acc; }, function (syncedDoc) {
                    return optionFold(acc, function () {
                        var _a;
                        return Some((_a = {},
                            _a[fieldName] = RefField({
                                doc: syncedDoc,
                                id: field.snapshot.id,
                            }),
                            _a));
                    }, function (acc) {
                        var _a;
                        return Some(__assign(__assign({}, acc), (_a = {}, _a[fieldName] = RefField({
                            doc: syncedDoc,
                            id: field.snapshot.id,
                        }), _a)));
                    });
                }));
            });
        });
    });
}
//# sourceMappingURL=filter-synced-fields.js.map